/**
 * JOnAS Configurator
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Jonas.java 15670 2008-10-27 10:41:47Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.tools.configurator;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;

import org.apache.log4j.Logger;
import org.ow2.jonas.tools.configurator.api.JonasConfigurator;

/**
 * This class contain the JonasConfigurator to use depends of the Jonas version.
 * It select dynamically the version to use.
 * 
 * @author remy bresson
 */
public class Jonas {

    /**
     * JOnAS major version
     */
    int majorVersion;

    /**
     * Interface used to pass configuration task to JOnAS.
     */
    protected JonasConfigurator jonasConfigurator;

    /**
     * Configuration logger
     */
    private static Logger logger = Logger.getLogger(Jonas.class);

    /**
     * Jonas constructor.
     * 
     * @param jonasRoot The path where is located the jonas binaries.
     */
    public Jonas(final String jonasRoot) {
        try {
            URL antTasks = new File(jonasRoot + File.separator + "lib" + File.separator + "common" + File.separator
                + "ow_jonas_ant.jar").toURI().toURL();
            URL owJonas = new File(jonasRoot + File.separator + "lib" + File.separator + "commons" + File.separator + "jonas"
                + File.separator + "ow_jonas.jar").toURI().toURL();
            logger.debug("Configuring jonasRoot : " + jonasRoot + " using ant library on URL : " + antTasks);

            ClassLoader search = new URLClassLoader(new URL[] {antTasks}, null);

            if (search.getResource("org/ow2/jonas/ant/JOnASBaseTask.class") != null) {
                this.majorVersion = 5;
            } else if (search.getResource("org/objectweb/jonas/ant/JOnASBaseTask.class") != null) {
                this.majorVersion = 4;
            } else {
                throw new Exception("Not a JOnAS 4 or JOnAS 5 ANT tasks JAR file");
            }

            ClassLoader self = this.getClass().getClassLoader();
            InputStream is = self.getResourceAsStream("configurator-impl.jar");
            File temp = File.createTempFile(Jonas.class.getName(), ".jar");
            temp.deleteOnExit();
            FileOutputStream fos = new FileOutputStream(temp);
            for (int length = is.available(); length > 0; length = is.available()) {
                byte[] b = new byte[length];
                length = is.read(b);
                fos.write(b, 0, length);
            }
            fos.close();
            ClassLoader instanciate = new URLClassLoader(new URL[] {antTasks, owJonas, temp.toURI().toURL()}, self);
            Class<? extends JonasConfigurator> instance = instanciate.loadClass(
                "org.ow2.jonas.tools.configurator.impl.Jonas" + this.majorVersion).asSubclass(JonasConfigurator.class);
            this.jonasConfigurator = instance.newInstance();
            instance.getMethod("setJonasRoot", String.class).invoke(this.jonasConfigurator, jonasRoot);
        } catch (Exception e) {
            throw new IllegalArgumentException("The file \"" + jonasRoot + "\" doesn't contain any JOnAS ANT tasks JAR file", e);
        }
    }

    /**
     * Get the jonas major version
     * 
     * @return the jonas major version
     */
    public int getMajorVersion() {
        return this.majorVersion;
    }

    /**
     * Get the jonas configurator
     * 
     * @return the jonasConfigurator
     */
    public JonasConfigurator getJonasConfigurator() {
        return this.jonasConfigurator;
    }
}
